#!/usr/bin/env python

import sys
import os
from sys import platform
import re

sizeDict = {
    "uint8" : 1,
    "int8" : 1,
    "char" : 1,
    "uint16" : 2,
    "int16" : 2,
    "uint32" : 4,
    "int32" : 4,
    "int64" : 8,
    "uint64" : 8,
    "int" : 4,
    "string" : 16,
}

cTypeDict = {
    "uint8" : "uint8_t",
     "int8" : "int8_t",
     "char" : "char",
     "string" : "char",
     "uint16" : "uint16_t",
     "int16" : "int16_t",
     "uint32" : "uint32_t",
     "int32" : "int32_t",
     "int64" : "int64_t",
     "uint64" : "uint64_t",
     "int" : "int",
     "float" : "float",
     "double" : "double",
     "enum" : "uint8_t",
     "flag" : "uint8_t"
 }

class CodeReplacer:
    def __init__(self):
        self.input =""
        self.blocks = {}
        self.blocksUsed = {}
        
    
    def loadText(self, text, regex=r"\*user-block-(.*?)-start\*(.*?)\*user-block-\1-end\*"):
        rx = re.compile(regex,re.DOTALL)

        for match in rx.finditer(text):
            token = match.groups(0)[0]
            block = match.group()
            self.blocks[token] = block
            self.blocksUsed[token] = False


    def insertBlocks(self, text, regex=r"\*user-block-(.*?)-start\*(.*?)\*?user-block-\1-end\*"):
        rx = re.compile(regex,re.DOTALL)
        
        for match in rx.finditer(text):
            token = match.groups(0)[0]
            if token in self.blocks:
                self.blocksUsed[token] = True
                text = text.replace(match.group(), self.blocks[token])
        
        
        return text

    def printDrops(self):
        
        for token, used in self.blocksUsed.items():
            if not used:
                print(" Code block dropped: " + token)
                print(self.blocks[token])

class TemplateHelper:
    def __init__(self):
        self.sizeDict = sizeDict
        self.cTypeDict = cTypeDict

    def camelCase(self, text):
        out =""
        cap = True 
        for char in text:
            if char != '_':
                if cap:
                    out+= char.upper()
                else:
                    out+= char.lower()
                cap = False
            else:
                cap = True
            
        return out
    
    def padAfter(self, text, spacing):
        ret = text
        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret 








#!/usr/bin/env python3
#
#@file device.py
#@brief device descripto
#@author Jason Berger
#@date 02/19/2019
#

import sys
import os
import yaml
import json
from mrtutils.mrtYamlHelper import *


sizeDict = {
    "uint8" : 1,
    "int8" : 1,
    "char" : 1,
    "uint16" : 2,
    "int16" : 2,
    "uint32" : 4,
    "int32" : 4,
    "int64" : 8,
    "uint64" : 8,
    "int" : 4,
    "string" : 16,
}

cNameDict = {
    "uint8" : "uint8_t",
     "int8" : "int8_t",
     "char" : "char",
     "string" : "char",
     "uint16" : "uint16_t",
     "int16" : "int16_t",
     "uint32" : "uint32_t",
     "int32" : "int32_t",
     "int64" : "int64_t",
     "uint64" : "uint64_t",
     "int" : "int",
     "float" : "float",
     "double" : "double",
     "enum" : "uint8_t",
     "flag" : "uint8_t"
 }

args = None
parser = None

class DevConfig:
    def __init__(self, node):
        self.field = 0
        self.val = 0 
        self.desc = ""
        self.regVals = []
        self.device = 0

        self.name = list(node.keys())[0]
        configItem = list(node.values())[0]
        if 'desc' in configItem:
            self.desc = configItem['desc']
        
        if 'registers' in configItem:
            for regNode in configItem['registers']:
                self.regVals.append(regNode)
        if 'regs' in configItem:
            for regNode in configItem['regs']:
                self.regVals.append(regNode)
    
    def getDesc(self,regVal, spacing =0):
        ret =""
        regName = list(regVal.keys())[0]
        regItem = list(regVal.values())[0]
        if type(regItem) is dict:
            if 'desc' in regItem:
                return "/* "+regItem['desc']+" */"
            else:
                ret+="/*"
                for key,value in regItem.items():
                    if not key == 'delay':
                        ret+= " "+key+": "+ str(value) +" ,"
                ret+=",*/"
                ret = ret.replace(",,", "")
        else:
            ret=""

        spaces = spacing - len(ret)
        ret+= (" " * spaces)
        
        return ret
    
    def getDelay(self, regVal, spacing = 0):
        regName = list(regVal.keys())[0]
        regItem = list(regVal.values())[0]
        ret =""
        spaces = 0
        if type( regItem) is dict:
            if 'delay' in regItem:
                ret+= "MRT_DELAY_MS({}); ".format(regItem['delay'])
                spaces = spacing - len(ret)
                ret+= (" " * spaces)
                ret+= " /* Delay for {} */ \\".format(regName)
                return ret

        return False

    
    def getDict(self):
        
        json_dict = {}
        for configReg in self.regVals:
            regName = list(configReg.keys())[0]
            regItem = list(configReg.values())[0]
            val = 0
            if regName in self.device.regs:
                curReg = self.device.regs[regName]
                if type( regItem) is dict:
                    val =0
                    for key,value in regItem.items():
                        if key in curReg.fieldDict:
                            curField = curReg.fieldDict[key]
                            if type(value) is str:
                                if value in curField.valDict:
                                    val = val | (curField.valDict[value].val << curField.offset) & curField.mask
                            else:
                                val = val | (value << curField.offset) & curField.mask
                    
                else: 
                    val = regItem

            json_dict[regName] = val 

        return json_dict    



class FieldVal:
    def __init__(self, node):
        self.field = 0
        self.val = 0 
        self.desc = ""

        self.name = list(node.keys())[0]
        valItem = list(node.values())[0]
        if 'val' in valItem:
            self.val = valItem['val']
        if 'desc' in valItem:
            self.desc = valItem['desc']
        if 'name' in valItem:
            self.desc = valItem['name']

    def getFieldValMacro(self, spacing = 0):
        ret = self.field.reg.device.prefix.upper() +"_"+self.field.reg.name.upper()+"_"+self.field.name.upper() +"_" + self.name.upper()
        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret
    
    def getFieldValMaskMacro(self, spacing = 0):
        ret = self.field.reg.device.prefix.upper() +"_"+self.field.reg.name.upper()+"_"+self.field.name.upper() +"_" + self.name.upper() + "_MASK"
        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret
    
    def getOffSetValue(self):
        return val << self.field.offset
    
    def getDict(self):
        json_dict ={"name": self.name, "val": self.val, "desc": self.desc}
        return json_dict
    
    def formatVal(self):
        if self.field.bitCount > 6 :
            return "x"+format(self.val, '02x')
        else:
            return "b"+format(self.val, '0'+str(self.field.bitCount)+'b')

 
class RegField:
    def __init__(self, node):
        self.reg = 0
        self.mask = 0XFFFFFFFF
        self.desc = ""
        self.vals = []
        self.valDict = {}
        self.isFlag = False
        self.bitCount = 0
        self.startBit =0
        self.flat = False

        if 'name' in node:
            self.name = node['name']
        if 'mask' in node:
            self.mask = node['mask']
        if 'bit' in node:
            self.mask = 1 << node['bit']
        if 'values' in node:
            for valNode in node['values']:
                newVal = FieldVal(valNode)
                self.addVal(newVal)
        if 'vals' in node:
            for valNode in node['vals']:
                newVal = FieldVal(valNode)
                self.addVal(newVal)
        if 'desc' in node:
            self.desc = node['desc']
        
        if self.getSize() == 1:
            self.isFlag = True
        
        self.bitCount = self.getSize()
        self.offset = self.getOffset()
        self.startBit = self.bitCount + self.offset

    def getSize(self):
        count = 0
        n = self.mask
        while (n): 
            count += n & 1
            n >>= 1
        return count 

    def getOffset(self):
        check = self.mask
        count = 0
        while not check & 1:
            check = check >> 1
            count+=1
    
        return count
    
    def addVal(self, fieldVal):
        fieldVal.field = self
        self.vals.append(fieldVal)
        self.valDict[fieldVal.name] = fieldVal

    def getFieldMaskMacro(self, spacing = 0):
        ret = self.reg.device.prefix.upper() +"_"+self.reg.name.upper()+"_"+self.name.upper() +"_FIELD_MASK"

        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret
    
    def getFieldOffsetkMacro(self, spacing = 0):
        ret = self.reg.device.prefix.upper() +"_"+self.reg.name.upper()+"_"+self.name.upper() +"_FIELD_OFFSET"

        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret

    def getFieldFlagMacro(self, spacing = 0):
        ret = self.reg.device.prefix.upper() +"_"+ self.reg.name.upper()+"_"+self.name.upper()

        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret

    def getToolTip(self):
        message = ""

        for val in self.vals:
            if self.bitCount > 6 :
                message +="\n\tx" + format(val.val, '02x') +" = " + val.desc
            else:
                message +="\n\tb" + format(val.val, '0'+str(self.bitCount)+'b') +" = " + val.desc
        
        return self.desc, message

    def getDict(self):
        val_arr = []
        for val in self.vals:
            val_arr.append(val.getDict())

        json_dict = { "mask": self.mask, "size": self.bitCount, "offset": self.offset, "desc": self.desc, "vals": val_arr}

        return json_dict
        

        

class DeviceReg:
    def __init__(self,name):
        self.name = name
        self.addr = 0
        self.type ="uint8"
        self.perm = "RW"
        self.desc = ""
        self.default = 0
        self.hasDefault = False
        self.fields = []
        self.fieldDict = {}
        self.flags = []
        self.size = 1
        self.device = 0
        self.hasFlags = False
        self.hasFields = False
        self.configs = {}
    
    def addField(self, field):
        field.reg = self
        if field.isFlag:
            self.hasFlags = True
        else:
            self.hasFields = True

        self.fieldDict[field.name] = field
        self.fields.append(field)
    
    def addDefaultField(self):

        fieldNode = {
            "name": self.name,
            "desc": self.desc
        }

        if self.size == 1:
            fieldNode['mask'] = 0xFF
        if self.size == 2: 
            fieldNode['mask'] = 0xFFFF
        if self.size == 4: 
            fieldNode['mask'] = 0xFFFFFFFF

        field = RegField(fieldNode)
        field.flat = True
        self.addField(field)
    
    def formatHex(self, val):
        return self.device.formatHex(val, self.size)

    def printAddr(self):
        val = "{0:#0{1}X}".format(self.addr,(self.device.addrSize *2) + 2)
        val = val.replace("X","x")
        return val
    
    def getNextfieldByStartBit(self, startBit):

        found = False
        for field in self.fields:
            if field.startBit == startBit:
                return field , field.bitCount
        
        search = startBit
        while search >= 0:
            search = search -1 
            for field in self.fields:
                if field.startBit == search:
                    return False , startBit - search
        
        return False, startBit

    
    
    def printFieldMap(self):
        ret =""
        i = self.size * 8
        while i > 0:
            field , nextStart = self.getNextfieldByStartBit(i)
            if field:
                ret = ret+"<td class=\"field\" colspan=\""+str(field.bitCount)+"\">" +field.name+"</td>\n"
                i -= field.bitCount
            else :
                ret = ret+"<td class=\"empty\" colspan=\""+ str( nextStart)+"\"></td>\n"
                i -= nextStart
        
        return ret
    
    
    def getAddrMacro(self, spacing = 0):
        ret = self.device.prefix.upper() +"_REG_"+self.name.upper()+"_ADDR"
        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret 
    
    def getDefaultMacro(self, spacing = 0):
        ret = self.device.prefix.upper() +"_"+self.name.upper()+"_DEFAULT"
        if spacing > 0:
            spaces = spacing - len(ret)
            ret = ret + (" " * spaces)
        return ret

    def getDict(self):
        field_dict = {}

        for field in self.fields:
            field_dict[field.name] = field.getDict();

        json_dict = {"name": self.name, "addr": self.addr, "type": self.type, "size": self.size * 8, "perm": self.perm.upper(), "default":self.default, "hasDefault":self.hasDefault, "desc": self.desc, "fields": field_dict}
        return json_dict


class Device:
    def __init__(self, name):
        self.name = name
        self.prefix = ""
        self.bus = "I2C"
        self.i2c_addr = 0
        self.addrSize =1
        self.aiMask = 0 #auto incrment mask 
        self.regs = {}
        self.datasheet =""
        self.desc=""
        self.digikey_pn =""
        self.smallestReg = 4 
        self.largestReg = 1
        self.configs = {}
        self.defaults = {}

    def addReg(self, reg):
        reg.device = self
        if reg.size < self.smallestReg:
            self.smallestReg = reg.size
        
        if reg.size > self.largestReg:
            self.largestReg = reg.size
        
        if reg.hasDefault:
            self.defaults[reg.name] = reg.default

        self.regs[reg.name] = reg
    
    def addConfig(self,config):
        config.device = self
        self.configs[config.name] = config

    def getConfigLine(self,configReg,spacing =0, macro = False):
        ret = ""
        regName = list(configReg.keys())[0]
        regItem = list(configReg.values())[0]

        if regName in self.regs:
            curReg = self.regs[regName]
            if(macro):
                ret = self.prefix.lower() + "_write_reg( (dev), &(dev)->"
            else:
                ret = self.prefix.lower() + "_write_reg( dev, &dev->"
            ret+= "m"+ self.camelCase(regName) + ", "
            if type( regItem) is dict:
                val =0
                for key,value in regItem.items():
                    if key in curReg.fieldDict:
                        curField = curReg.fieldDict[key]
                        if type(value) is str:
                            if value in curField.valDict:
                                val = val | (curField.valDict[value].val << curField.offset) & curField.mask
                        else:
                            val = val | (value << curField.offset) & curField.mask
                
                ret+= self.formatHex(val, curReg.size)
            else: 
                ret+= self.formatHex(regItem, curReg.size)

            ret+=");"
        spaces = spacing - len(ret)
        ret+= (" "*spaces)
        return ret

    def getConfigRegVal(self,configReg):
        
        regName = list(configReg.keys())[0]
        regItem = list(configReg.values())[0]
        val = 0
        if regName in self.regs:
            curReg = self.regs[regName]
            if type( regItem) is dict:
                val =0
                for key,value in regItem.items():
                    if key in curReg.fieldDict:
                        curField = curReg.fieldDict[key]
                        if type(value) is str:
                            if value in curField.valDict:
                                val = val | (curField.valDict[value].val << curField.offset) & curField.mask
                        else:
                            val = val | (value << curField.offset) & curField.mask
                
            else: 
                val = regItem

        return regName, val
        

    def formatHex(self, val, size):
        val = "{0:#0{1}X}".format(val,(size *2) + 2)
        val = val.replace("X","x")
        return val

    def camelCase(self, text):
        out =""
        cap = True 
        for char in text:
            if char != '_':
                if cap:
                    out+= char.upper()
                else:
                    out+= char.lower()
                cap = False
            else:
                cap = True
            
        return out

    def parseYAML(self,yamlFile):
        data = open(yamlFile)
        objDevice = yaml.load(data , Loader=yaml.FullLoader)

        if 'name' in objDevice:
            self.name = objDevice['name']
        
        if 'prefix' in objDevice:
            self.prefix = objDevice['prefix']
        
        if 'bus' in objDevice:
            self.bus = objDevice['bus']
        
        if 'digikey_pn' in objDevice:
            self.digikey_pn = objDevice['digikey_pn']

        if 'mfg_pn' in objDevice:
            self.mfg_pn = objDevice['mfg_pn']
        
        if 'desc' in objDevice:
            self.desc = objDevice['desc']
        
        if 'description' in objDevice:
            self.desc = objDevice['description']
        
        if 'i2c_addr' in objDevice:
            self.i2c_addr = objDevice['i2c_addr']

        if 'datasheet' in objDevice:
            self.datasheet = objDevice['datasheet']
        
        if 'addr_size' in objDevice:
            self.addr_size = objDevice['addr_size']
        
        if 'ai_mask' in objDevice:
            self.aiMask = objDevice['ai_mask'] #int(objDecive['ai_mask'],0)
        


        if 'registers' in objDevice:
            regs = yamlNormalizeNodes( objDevice['registers'], 'reg_name','addr')
            for regItem in regs:
                newReg = DeviceReg(regItem['reg_name'])

                if 'addr' in regItem:
                    newReg.addr = regItem['addr'] #int(regItem['addr'],0) 
                if 'type' in regItem:
                    newReg.type = regItem['type'].replace("_t","")
                    newReg.size = sizeDict[newReg.type.replace("_t","")]
                if 'size' in regItem:
                    newReg.size = regItem['size']
                if 'perm' in regItem:
                    newReg.perm = regItem['perm'].upper()
                if 'desc' in regItem:
                    newReg.desc = regItem['desc']
                if 'name' in regItem:
                    newReg.desc = regItem['name']
                if 'default' in regItem:
                    newReg.default = regItem['default']
                    newReg.hasDefault = True

                self.addReg(newReg)      
        
        if 'fields' in objDevice:
            for propNode in objDevice['fields']:
                if type(propNode) is dict:
                    regName = list(propNode.keys())[0]
                    propItem = list(propNode.values())[0]
                    if regName in self.regs:
                        curReg = self.regs[regName]  
                        fieldNodes = yamlNormalizeNodes(propItem, 'name', 'mask')
                        for fieldNode in fieldNodes:
                            newField = RegField(fieldNode)
                            curReg.addField(newField)
                else:
                    regName = propNode
                    if regName in self.regs:
                        self.regs[regName].addDefaultField()
        
        if 'configs' in objDevice:
            for configNode in objDevice['configs']:
                configItem = DevConfig(configNode)
                self.addConfig(configItem)
        
        if 'configurations' in objDevice:
            for configNode in objDevice['configurations']:
                configItem = DevConfig(configNode)
                self.addConfig(configItem)

        #if a register has no fields but has a default value, make it a flat reg/field
        for key,value in self.regs.items():
            if len(value.fields) == 0 and value.hasDefault:
                value.addDefaultField() 

        print("Parsed device: " + self.name )
        print( "registers: " + str(len(self.regs)))

    def getDict(self):

        reg_dict ={}
        cfg_dict = {}
        for key, reg in self.regs.items():
            reg_dict[key] = reg.getDict()
        
        cfg_dict['Default'] = self.defaults
        for key, cfg in self.configs.items():
            cfg_dict[key] = cfg.getDict()

        json_dict = {"name": self.name, "regs": reg_dict, "configs": cfg_dict, "currentVals": self.defaults, "smallestReg" : self.smallestReg * 8 , "largestReg": self.largestReg * 8}

        return json_dict
    
    def printJSON(self):
        return json.dumps(self.getDict());





        



        


#!/usr/bin/env python3
#
#@file gatt_profile
#@brief Gatt Profile descriptor
#@author Jason Berger
#@date 02/19/2019
#

import sys
import os
import yaml
import re
import json
import datetime
import urllib.request
from mrtutils.mrtYamlHelper import *
import xml.etree.ElementTree as ET 


sizeDict = {
    "uint8" : 1,
    "int8" : 1,
    "char" : 1,
    "uint16" : 2,
    "int16" : 2,
    "uint32" : 4,
    "int32" : 4,
    "int64" : 8,
    "uint64" : 8,
    "int" : 4,
    "string" : 16,
    "utfs8": 16
}

svcIconDict = {
    "Device" : "fa-desktop",
    "Battery" : "fa-battery-three-quarters",
    "Environmental" : "fa-leaf"
}

charIconDict = {
    "Level" : "fa-battery-half",
    "Firmware_Revision" : "fa-code",
    "Temperature" : "fa-thermometer-empty",
    "Humidity" : "fa-tint",
}


typeDict ={
    "utf8s" : "string",
    "sint16" : "uint16",
    "sint" : "int",
    "sint8" : "int8"
}

attrDict = {
    "description" : "desc",
    "characteristics" : "chars",
    "values" : "vals",
    "value" : "val",
    "permissions" : "perm"
}

def getXml(url):
    headers={ 'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36'}
    req = urllib.request.Request(url,data=None, headers = headers)
    file = urllib.request.urlopen(req)
    data = file.read()
    file.close()
    root = ET.fromstring(data.decode("utf-8"))

    return root

def insert_dash(string, index):
    return string[:index] + '-' + string[index:]

def uuidStr(val , short = False):
    arr =[]
    if type(val) != str:
        val = "0000%0.4X-0000-1000-8000-00805f9b34fb" % val
        ret = val
    else:  
        val = val.replace('-','')
        arr = [val[i:i+2] for i in range(0, len(val), 2)]
        ret = ''.join(arr)

        if len(ret) > 4:
            ret = insert_dash(ret, 8)
            ret = insert_dash(ret, 13)
            ret = insert_dash(ret, 18)
            ret = insert_dash(ret, 23)
    
    if short:
        ret = ret[4:8]

    return  ret.lower() 

def setIfEmpty(obj, field, new):
    if obj.__dict__[field] is None and new is not None:
        obj.__dict__[field] = new.text.strip()

class GattValue:
    def __init__(self):
        self.name = ""
        self.value = 0
        self.desc = None
    
    def parseYaml(self, node):
        yamlGetAttributes(node, attrDict, self)

    def getDict(self):

        json_dict = { "name": self.name, "val": self.value }

        return json_dict

class GattCharacteristic(object):
    def __init__(self):
        self.name = ""
        self.uri = ""
        self.desc= None
        self.vals = []
        self.isEnum = False 
        self.isMask = False
        self.valsFormat = "0x%0.2X"
        self.type = 'uint8'
        self.isStandard = False
        self.perm = None
        self.url =""
        self.uuid = None
        self.uuidType = "e128Bit"
        self.icon = ''
        self.coef = 1
        self.unit = ''
        self.arrayLen = 1
        self.nextVal = 0
    
    def uuidArray(self):
        val = self.uuid 
        arr =[]
        if type(val) != str:
            val = "%0.4X" % val
            self.uuidType ="e16Bit"
        
        val = val.replace('-','')
        arr = [val[i:i+2] for i in range(0, len(val), 2)]
        ret = ', 0x'.join(arr[::-1])

        return "0x"+ ret 
    
    def size(self):
        return sizeDict[self.type] * self.arrayLen
    
    def uuidStr(self):
        
        if type(self.uuid) is str:
            return self.uuid.split('-')[1]
        else:
            return "%0.4X" % self.uuid
    
    def props(self):
        ret = ""

        props = []
        if 'r' in self.perm.lower():
            props.append('MRT_GATT_PROP_READ')
        if 'w' in self.perm.lower():
            props.append('MRT_GATT_PROP_WRITE')
        if 'n' in self.perm.lower():
            props.append('MRT_GATT_PROP_NOTIFY')
        if 'i' in self.perm.lower():
            props.append('MRT_GATT_PROP_INDICATE')
        
        if len(props) > 0:
            return " | ".join(props)
        else:
            return "MRT_GATT_PROP_NONE"


    
    def loadUri(self,uri):
        print("Pulling " + uri)
        self.url = "https://www.bluetooth.com/wp-content/uploads/Sitecore-Media-Library/Gatt/Xml/Characteristics/" + uri + ".xml"
        root = getXml(self.url)
        self.uri = uri
        self.isStandard = True

        self.name = root.find('./Value/Field').attrib['name'].replace(' ', '_')
        self.uuid = int(root.attrib['uuid'], 16)
        sigType = root.find('./Value/Field/Format').text
        if sigType in typeDict:
            sigType = typeDict[sigType]
        
        
        self.type = sigType

        setIfEmpty(self, 'desc',root.find('./InformativeText/Abstract'))
        setIfEmpty(self, 'desc',root.find('./InformativeText/Summary') )
        setIfEmpty(self, 'desc',root.find('./InformativeText'))
        setIfEmpty(self, 'desc',root.find('./Value/Field/InformativeText/Abstract'))
        setIfEmpty(self, 'desc',root.find('./Value/Field/InformativeText/Summary') )
        setIfEmpty(self, 'desc',root.find('./Value/Field/InformativeText'))
        if self.desc is None:
            self.desc = "No Descriptions"

        self.desc = self.desc.replace('\n','').replace('\t','')
    
    def loadServiceUri(self, uri):
        url = "https://www.bluetooth.com/wp-content/uploads/Sitecore-Media-Library/Gatt/Xml/Services/" + uri + ".xml"
        root = getXml(url)
 
        xmlChar = root.find('./Characteristics/Characteristic[@type=\"' + self.uri + '\"]')

        sigPerm = ""
        if xmlChar.find('./Properties/Read').text in ['Mandatory','Optional']:
            sigPerm += "R"
        if xmlChar.find('./Properties/Write').text in ['Mandatory','Optional']:
            sigPerm += "W"
        if xmlChar.find('./Properties/Notify').text in ['Mandatory','Optional']:
            sigPerm += "N"
        if xmlChar.find('./Properties/Notify').text in ['Mandatory','Optional']:
            sigPerm += "I"
        self.perm = sigPerm

        if self.name in svcIconDict:
                self.icon = charIconDict[self.name]

    
    def parseYaml(self, node):
        
        if 'uri' in node:
            self.loadUri(node['uri'])

        yamlGetAttributes(node, attrDict, self)

        if self.type in ['flag','flags','mask','bits']:
            self.isMask = True

        if self.type in ['enum','enums']:
            self.isEnum = True
        
        m = re.search('\*([0-9]*)', self.type)
        if(m):
            if(m.group(1) != ''):
                self.arrayLen = int(m.group(1))
            self.type = self.type[0:m.start()]

        if 'vals' in node:
            valNodes = yamlNormalizeNodes( node['vals'], 'name','desc')
            for valNode in valNodes:
                newVal = GattValue()
                newVal.parseYaml(valNode)
                self.addVal(newVal)
    
    def addVal(self, val):

        if self.isMask:
            val.value = 1 << self.nextVal
            strType = 'uint8'
            if len(self.vals) > 8:
                self.valsFormat = "0x%0.4X"
                strType = 'uint16'
            if len(self.vals) > 16:
                self.valsFormat = "0x%0.8X"
                strType = 'uint32'
            if len(self.vals) > 32:
                print( "Error maximum flags per field is 32")
            self.type = strType
        else:
            val.value = self.nextVal 
            self.type = 'uint8'
        
        self.nextVal = self.nextVal +1
        self.vals.append(val)
    
    def getDict(self):
        val_arr = [] #array of gattValues for enums/masks

        for val in self.vals:
            val_arr.append(val.getDict())


        json_dict = { "name": self.name, "id": self.name.replace(' ', '_'), "size": sizeDict[self.type], "arrayLen": self.arrayLen, "uuid": uuidStr(self.uuid), "short_uuid": uuidStr(self.uuid, True), "uri":self.uri ,"url": self.url, "type": self.type, "unit": self.unit, "coef":self.coef, "icon" : self.icon, "uuid_type": self.uuidType,  "perm": self.perm, "desc": self.desc.rstrip()  , "vals": val_arr}

        if(self.isEnum):
            json_dict['type'] = 'enum'

        if(self.isMask):
            json_dict['type'] = 'mask'

        return json_dict

class GattService(object):
    def __init__(self):

        #load uri first so we can override attributes as needed 
        
        self.name = "unnamed"
        self.chars = []
        self.desc = None
        self.isStandard = False
        self.url =""
        self.uri = None
        self.nextUuid = 0
        self.uuidType ="e128Bit"
        self.profile = ''
        self.icon = ''
    
    def loadUri(self, uri):
        print("Pulling " + uri)
        self.url = "https://www.bluetooth.com/wp-content/uploads/Sitecore-Media-Library/Gatt/Xml/Services/" + uri + ".xml"
        root = getXml(self.url)
        self.isStandard = True
        self.uri = uri

        self.name = root.attrib['name'].replace(' ', '_')
        self.uuid = int(root.attrib['uuid'],16)
        self.uuidType = 'e16Bit'
        setIfEmpty(self, 'desc',root.find('./InformativeText/Abstract'))
        setIfEmpty(self, 'desc',root.find('./InformativeText/Summary'))


        for xmlChar in root.findall("./Characteristics/Characteristic"):
            if xmlChar.find('./Requirement').text == 'Mandatory':
                newChar = GattCharacteristic()
                newChar.loadUri(xmlChar.attrib['type'])
                newChar.loadServiceUri(self.uri)
                self.chars.append(newChar)
        
        self.desc = self.desc.replace('\n','').replace('\t','').strip()


    def uuidArray(self):
        val = self.uuid 
        arr =[]
        if type(val) != str:
            val = "%0.4X" % val
            self.uuidType ="e16Bit"
        
        val = val.replace('-','')
        arr = [val[i:i+2] for i in range(0, len(val), 2)]
        ret = ', 0x'.join(arr[::-1])

        return "0x"+ ret 

    def parseYaml(self, node):
        
        uuidSplit = []
        if 'uri' in node:
            self.loadUri(node['uri'])
            self.nextUuid = self.uuid + 1
            yamlGetAttributes(node, attrDict, self)
            if self.name in svcIconDict:
                self.icon = svcIconDict[self.name]
        else:
            yamlGetAttributes(node, attrDict, self)
            uuidSplit = self.uuid.split('-')
            self.nextUuid = int(uuidSplit[1], 16) + 1
        
        if 'chars' in node:
            chars = node['chars']
            charNodes = yamlNormalizeNodes( node['chars'], 'name', 'uri', False)
            for charNode in charNodes:
                newChar = GattCharacteristic()
                newChar.parseYaml(charNode)
                if self.uri is not None:
                    newChar.loadServiceUri(self.uri)
                if newChar.uuid is None:                        #if no uuid is set, increment previous
                    uuidSplit[1] = "%0.4X" % self.nextUuid
                    newChar.uuid = '-'.join(uuidSplit)
                    self.nextUuid += 1
                if type(newChar.uuid) != str and self.uri is None:   #if it is a  16 bit uuid without a uri, it incremements from the next uuid of the service
                    uuidSplit[1] = "%0.4X" % newChar.uuid
                    self.nextUuid = newChar.uuid + 1
                    newChar.uuid = '-'.join(uuidSplit)
                self.addChar(newChar)
    
    def addChar(self, char):
        self.chars.append(char)

    def getDict(self):
        char_arr = []

        for char in self.chars:
            char_arr.append(char.getDict())
        
        json_dict = { "name": self.name, "id": self.name.replace(' ', '_'), "uuid": uuidStr(self.uuid),  "short_uuid": uuidStr(self.uuid, True), "uuid_type": self.uuidType, "desc": self.desc, "icon" : self.icon, "uri": self.uri, "url": self.url, "characteristics": char_arr}

        return json_dict
             

class GattProfile(object):
    def __init__(self):
        self.name= "unnamed"
        self.services = []
        self.genTime = datetime.datetime.now().strftime("%m/%d/%y")
    
    def nrfServices(self, uuidType):

        serviceItems =[]
        for service in self.services:
            if service.uuidType == uuidType:
                item = '\t\t"{0}" : {{\n\t\t\t"name":"{1}",\n\t\t\t}}'.format(uuidStr(service.uuid), service.name)
                serviceItems.append(item)
        
        ret = ',\n'.join(serviceItems)
        if ret != "":
            ret = "\n" + ret 
        
        return ret
    
    def nrfChars(self,uuidType):
        charItems =[]
        for service in self.services:
            for char in service.chars:
                if char.uuidType == uuidType:
                    item = '\t\t"{0}" : {{\n\t\t\t"name":"{1}",\n\t\t\t"format":"{2}"\n\t\t\t}}'.format(uuidStr(char.uuid), char.name, "TEXT" if (char.type == 'string') else "NO_FORMAT" )
                    charItems.append(item)
        
        ret = ',\n'.join(charItems)
        if ret != "":
            ret = "\n" + ret
        
        return ret

    def parseYaml(self, file):
        data = open(file)
        nodeProfile = yaml.load(data, Loader = yaml.FullLoader)

        yamlGetAttributes(nodeProfile, attrDict, self)

        serviceNodes = yamlNormalizeNodes( nodeProfile['services'], 'name','uri')
        for serviceNode in serviceNodes:
            newService = GattService()
            newService.parseYaml(serviceNode)
            self.addService(newService)

    def addService(self, service):
        service.profile = self
        self.services.append(service)
    
    def getDict(self):
        svc_arr = []

        for svc in self.services:
            svc_arr.append(svc.getDict())
        
        json_dict = { "name": self.name,  "services": svc_arr}
        return json_dict
             




if __name__ == "__main__":
    prof = GattProfile()
    prof.parseYaml('templates/ble/gatt_descriptor_template.yml')
    





        



        


from update_notipy import update_notify
import pkg_resources

__version__ = pkg_resources.get_distribution("mrtutils").version

def updateCheck():
    update_notify('mrtutils', __version__).notify()

#    ┌───────────────────────────────────────────┐
#    │                                           │
#    │   Update available 0.1.0 → 0.1.2          │
#    │   Run pip install -U pkg-info to update   │
#    │                                           │
#    └───────────────────────────────────────────┘



from mrtutils.modsync import Repo, RepoDirectory, Submodule
from mrtutils.mrtTemplateHelper import *
from mrtutils.mrtYamlHelper import *
from mrtutils.device import *
from mrtutils.gatt_profile import *
from mrtutils.updates import *



#!/usr/bin/python3

import ctypes


#
# class disable_file_system_redirection:
#     _disable = ctypes.windll.kernel32.Wow64DisableWow64FsRedirection
#     _revert = ctypes.windll.kernel32.Wow64RevertWow64FsRedirection
#     def __enter__(self):
#         self.old_value = ctypes.c_long()
#         self.success = self._disable(ctypes.byref(self.old_value))
#     def __exit__(self, type, value, traceback):
#         if self.success:
#             self._revert(self.old_value)


import sys
import os
from sys import platform
import os.path
import re
import subprocess
import markdown
from threading import Thread
from urllib.request import urlopen

def getGitFileText(url, file):
    txt=""
    try:
        str = 'git archive --remote=' + url +' HEAD '+ file + ' | tar -xOf - ' + file
        txt = subprocess.check_output(str,shell=True, encoding='utf8')
    except subprocess.CalledProcessError as e:
        print(" Repo at " + url+ " does not contain " + file)

    return txt

def getBitbucketFile(account, repo, file):
    txt=""
    # try:
    response = urlopen("https://bitbucket.org/" + account + "/" + repo + "/raw/master/" + file)
    html_content = response.read()
    encoding = response.headers.get_content_charset('utf-8')
    txt = html_content.decode(encoding)
    # except:
    #     print("bitbucket.org/" + account + "/" + repo + "/raw/master/" + file + "  not found")
    
    return txt

class Submodule:
    def __init__(self, path, url):
        self.url = url

        self.git_url = url
        self.path = path
        self.nodes = path.split('/')
        self.name = self.nodes[-1]
        self.exists = False
        self.requirements = []
        self.readme="x"
        self.repo_name = url.split('/')[-1].replace(".git","")


    def checkFor(self,mods):
        global relativePath
        for mod in mods:
            if(self.name == mod.name):
                self.exists = True

    def getReadMe(self):
        if self.url.find("bitbucket"):
            self.readme = getBitbucketFile('uprev',self.repo_name, 'README.md')
        else:
            self.readme = getGitFileText('ssh://' + self.url.replace(':','/'), 'README.md')

        if(self.readme == ""):
            self.readme = "# No README.md is available for this module"


    def gatherRequirements(self):
            self.requirements = []
            #read in all modules
            regex = re.compile(r'>Requires: (.*?)')
            reqs = regex.findall(self.readme)
            for req in requirements:
                self.requirements.append(req[1])


class RepoDirectory:
    def __init__(self, name,lvl):
        self.name = name
        self.dirs = {}
        self.mods = {}
        self.mod = Submodule('','')
        self.isMod = False
        self.lvl = lvl

    def setMod(self,mod):
        self.mod = mod
        self.isMod = True


    def add(self, mod, lvl):

        nodes = mod.path.split('/')[lvl:]

        #print nodes[lvl:]

        if(len(nodes) > 1):
            node = nodes[0]
            if not node in self.dirs:
                self.dirs[node] = RepoDirectory(node, lvl)
            self.dirs[node].add(mod,lvl+1)
            if node in self.mods:
                self.dirs[node].setMod(self.mods[node])
                del self.mods[node]
        else:
            if mod.name in self.dirs:
                self.dirs[mod.name].set(mod)
            else:
                self.mods[mod.name] = mod

    def printout(self,lvl):
        start =""
        for i in range(lvl):
            start = start+"   "
            sys.stdout.write("---")

        sys.stdout.write(self.name + "\n")

        for mod in self.mods.values():
            sys.stdout.write(start + "+" +mod.name +"\n")

        for dir in self.dirs.values():
            dir.printout(lvl+1)


class Repo:
    def __init__(self, path):
        self.url = path
        self.path = path
        self.mods =[]
        self.isRemote = True
        self.dir = RepoDirectory("root",0)
        self.isBitbucket = False
        nodes = self.url.split('/')
        self.account = nodes[-2]
        self.name = nodes[-1].replace(".git","")

    def setRelativePath(self,path):
        self.relativePath = path
        self.isRemote = False;

    def getSubModules(self):
        data = ""
        if self.isRemote:
            if(self.isBitbucket):
                data = getBitbucketFile(self.account, self.name, '.gitmodules').replace('\r', '').replace('\n', '').replace('\t', '')
            else:
                data = getGitFileText(self.url,'.gitmodules').replace('\r', '').replace('\n', '').replace('\t', '')
            
            #read in all modules
            regex = re.compile(r'\[(.*?)].*?path = (.*?)url = (.*?\.git)')
            modules = regex.findall(data)
            for mod in modules:
                if(mod[1] != "Config"):
                    self.mods.append(Submodule(mod[1], mod[2]))
        else:
            if os.path.isfile(self.path + "/.gitmodules"):
                file = open(self.path + "/.gitmodules", "r")
                data = file.read().replace('\r', '').replace('\n', '').replace('\t', '')
                #read in all modules
                regex = re.compile(r'\[(.*?)].*?path = (.*?)url = (.*?\.git)')
                modules = regex.findall(data)
                for mod in modules:
                    self.mods.append(Submodule(mod[1], mod[2]))

    def getReadMe(self):
        if self.isRemote:
            if(self.isBitbucket):
                nodes = self.url.split('/')
                name = nodes[-1]
                account = nodes[-2]
                data = getBitbucketFile(self.account, self.name, 'README.md')
            else:
                data = getGitFileText(self.url, 'README.md')

        return data

    def fetchReadmes(self):
        print ("Fetching README.md from modules")
        for mod in self.mods:
            mod.getReadMe()

    def addSubModule(self,mod):
        prev_path = os.getcwd()
        os.chdir(self.path)
        print("Adding " + mod.git_url)
        subprocess.check_output(['git','submodule','add', '-f',mod.git_url,  self.relativePath + mod.path] )
        os.chdir(prev_path)
        mod.exists = True

    def removeSubModule(self,mod):
        prev_path = os.getcwd()
        os.chdir(self.path)
        subprocess.check_output(['git','rm', '-f', self.relativePath + mod.path ])
        os.chdir(prev_path)
        mod.exists = False



#!/usr/bin/env python

import sys
import os
from sys import platform
import yaml

##  Object can be defined a few ways, this function will normalize them
##
##   #0  {keykey: name, field1: val1, field2: val2} # this is what we want
##   #1  name: { field1: val1, field2: val2} = { keyKey: name,  field1: val1, field2: val2}
     #2  name: value                         = { keyKey: name, valKey: value}
     #3  name                                 = {keyKey: name, valKey: name}
## 
##
def yamlNormalizeNode(node, keyKey, valKey,allowSingle):
    retVal = {}

    if type(node) is dict: 
        #if the node is a dict with multiple values, it is a #0, already normalized
        if(len(node) > 1):
            return node

        #otherwise node is either a #1 or #2
        #get key/value
        nodeKey = list(node.keys())[0]
        nodeVal = list(node.values())[0]
        #if the dict has 1 key/value and the value is a dict we have a #1
        if type(nodeVal) is dict: 
            nodeVal[keyKey] = nodeKey
            return nodeVal
        else:
            if(allowSingle):
                retVal[keyKey] = nodeKey
                retVal[valKey] = nodeVal
                return retVal
            else:
                return node
    
    else:
        retVal[keyKey] = node 
        retVal[valKey] = node
        return retVal
    return node

def yamlNormalizeNodes(nodes, keyKey, valKey , allowSingle = True):

    retNodes = []
    for node in nodes:
        newNode = yamlNormalizeNode(node ,keyKey, valKey, allowSingle)
        retNodes.append(newNode)
    
    return retNodes

def yamlGetAttributes(node, attrDict, obj):

    for key, value in node.items():
        if type(value) is not list:
            if key in attrDict:
                newKey = attrDict[key]
                obj.__dict__[newKey] = value 
            else:
                obj.__dict__[key] = value

    

